package com.example.gitserver.controller;

import com.example.gitserver.common.ResultData;
import com.example.gitserver.service.GitService;
import com.example.gitserver.utils.GitUtil;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.io.IOException;
import java.util.List;


@RestController
@RequestMapping("/git")
public class GitController {

    Logger logger = LoggerFactory.getLogger(GitController.class);

    @Autowired
    private GitService gitService;

   /* @RequestMapping("/pull")
    public String pull(){
        String localPath="D:/gitRepositry/dev";
        String remotePath="http://git.mes123.com/root/zh-ws.git";
        String username="leimingming";
        String password="leiming2012";
        GitUtil gitUtil=new GitUtil(localPath,remotePath,username,password);

           // gitUtil.create();
        try {
           // gitUtil.pull("master");
           // gitUtil.cloneRepository(remotePath,"dev",localPath,username,password);
           // List<String> branchList= gitUtil.branchList("D:/gitRepositry/dev");
           // System.out.println(branchList.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "ok";
    }
*/
    @GetMapping(value = "/clone",produces = {"application/json;charset=UTF-8"})
    public ResultData clone(@RequestParam("remotePath") String remotePath,
                            @RequestParam("remotePath") String localPath,
                            @RequestParam("remotePath") String user,
                            @RequestParam("remotePath") String pwd,
                            @RequestParam(value = "remotePath",required = false) String branch){
       //判断本地路径是否被占用
       File file= GitUtil.initFile(localPath);
       if(null==file){
           return ResultData.fail("201","本地文件创建失败！！！");
       }else{
           try {
               gitService.clone(remotePath,file,user,pwd,branch==null?"master":branch);
               return ResultData.success("clone git 成功！");
           } catch (GitAPIException e) {

               logger.error("clone git 失败 ！！！",e);
               e.printStackTrace();
              return ResultData.fail("202","clone git 失败 ！！！");
           }
       }
    }


    @GetMapping(value = "/pull",produces = {"application/json;charset=UTF-8"})
    public ResultData pull(@RequestParam("localPath") String localPath,
                           @RequestParam("branchName") String branchName,
                           @RequestParam("user") String user,
                           @RequestParam("pwd") String pwd){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
                gitService.pull(localPath,branchName,user,pwd);
                return ResultData.success(branchName+"分支拉取成功！");
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }


    @GetMapping(value = "/checkout",produces = {"application/json;charset=UTF-8"})
    public ResultData checkout(@RequestParam("localPath") String localPath,@RequestParam("branchName") String branchName){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
                gitService.checkout(localPath,branchName);
                return ResultData.success(branchName+"分支检出成功！");
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }




    @GetMapping(value = "/getRemoteBranchList",produces = {"application/json;charset=UTF-8"})
    public ResultData getRemoteBranchList(@RequestParam("localPath") String localPath){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
               List<String> names= gitService.getRemoteBranchList(localPath);
                return ResultData.success("获取成功！",names);
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }



    @GetMapping(value = "/getLocalBranchList",produces = {"application/json;charset=UTF-8"})
    public ResultData getLocalBranchList(@RequestParam("localPath") String localPath){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
                List<String> names= gitService.getLocalBranchList(localPath);
                return ResultData.success("获取成功！",names);
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }



    @GetMapping(value = "/changeLocalBranch",produces = {"application/json;charset=UTF-8"})
    public ResultData changeLocalBranch(@RequestParam("localPath") String localPath,@RequestParam("branchName") String branchName){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
                gitService.changeLocalBranch(localPath,branchName);
                return ResultData.success("切换成功！");
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }


    @GetMapping(value = "/commit",produces = {"application/json;charset=UTF-8"})
    public ResultData commit(@RequestParam("localPath") String localPath,
                             @RequestParam("branchName") String branchName,
                             @RequestParam("desc")String desc){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
                gitService.changeLocalBranch(localPath,branchName);
                gitService.commit(localPath,desc);
                return ResultData.success("提交成功！");
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }



    @GetMapping(value = "/push",produces = {"application/json;charset=UTF-8"})
    public ResultData push(@RequestParam("localPath") String localPath,
                           @RequestParam("branchName") String branchName,
                           @RequestParam("user") String user,
                           @RequestParam("pwd") String pwd){
        //判断本地路径是否存在
        File file= new File(localPath);
        if(!file.exists()){
            return ResultData.fail("201","本地文件路径不存在！！！");
        }else{
            try {
                gitService.push(localPath,branchName,user,pwd);
                return ResultData.success("提交远程仓库成功！");
            } catch (GitAPIException | IOException e) {
                logger.error("操作失败！！！",e);
                e.printStackTrace();
                return ResultData.fail("202","操作失败！！！");
            }
        }
    }




}
