package com.example.gitserver.utils;


import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.ListBranchCommand;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.InvalidRemoteException;
import org.eclipse.jgit.api.errors.TransportException;
import org.eclipse.jgit.internal.storage.file.FileRepository;
import org.eclipse.jgit.lib.Constants;
import org.eclipse.jgit.lib.Ref;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public final class GitUtil {

    Logger log= LoggerFactory.getLogger(GitUtil.class);

    private String localPath, localGitPath, remotePath;
    private Repository localRepository;
    private String username;
    private String password;
    private Git git;

    public GitUtil(String localPath, String remotePath,String username ,String password) {
        this.username = username;
        this.password = password;
        this.localPath = localPath;
        this.remotePath = remotePath;
        this.localGitPath = this.localPath + "/.git";
        try {
            localRepository = new FileRepository(localGitPath);
        } catch (IOException e) {
            e.printStackTrace();
        }
        git = new Git(localRepository);

    }

    /**
     * 创建本地仓库
     *
     * @throws IOException
     */
    public void create() throws IOException {
        Repository repository = new FileRepository(localGitPath);
        repository.create();
        log.info("create success");
    }

    /**
     * clone克隆远程分支到本地
     *
     * @param branchName
     * @throws GitAPIException
     */
    public void cloneBranch(String branchName) throws GitAPIException {
        Git.cloneRepository()
                .setURI(remotePath)
                .setBranch(branchName)
                .setDirectory(new File(localPath)).call();
        log.info("clone success");
    }

    /**
     * pull远程代码
     *
     * @param branchName 远程分支名称
     * @throws Exception
     */
    public void pull(String branchName) throws Exception {
        git.pull().setRemoteBranchName(branchName).call();
        log.info("pull success");
    }

    /**
     * 将单个文件加入Git
     *
     * @param fileName 添加文件名
     * @throws Exception
     */
    public void add(String fileName) throws Exception {
        File myFile = new File(localPath + fileName);
        myFile.createNewFile();
        git.add().addFilepattern(fileName).call();
        log.info("add success");
    }

    /**
     * 将增加的所有文件加入Git
     *
     * @throws Exception
     */
    public void addAll() throws Exception {
        git.add().addFilepattern(".").call();
        log.info("add success");
    }


    /**
     * 提交文件
     *
     * @param message 提交备注
     * @throws Exception
     */
    public void commit(String message) throws Exception {
        git.commit().setMessage(message).call();
        log.info("commit success");
    }

    /**
     * 同步远程仓库
     *
     * @throws Exception
     */
    public void push() throws Exception {
        git.push().setCredentialsProvider(new UsernamePasswordCredentialsProvider(this.username, this.password)).call();
        log.info("push success");
    }


    public void checkOutBranch(String localPath) throws IOException {
        //-   refs/heads/test   refs/remotes/origin/dev_test_dev02   refs/remotes/origin/test
       // Git git =this.openGit(localPath);

        try {
            Git git = Git.open(new File("E:/repo"));
            List<Ref> list = git.branchList().setListMode(ListBranchCommand.ListMode.ALL).call();//ALL -a 所有分支  REMOTE -r 远程分支
            System.out.println(list);
            //切换远端分支
//            Ref ref= git.checkout().setCreateBranch(true)//创建新分支
//            .setName("dev_test_dev02")
//            .setStartPoint("origin/dev_test_dev02")//对应于起点选项
//            .call();
//            System.out.println(ref);
            //切换本地分支
            Ref ref = git.checkout().setCreateBranch(false).setName("test").call();
            System.out.println(ref);

        } catch (Exception e) {
            System.out.println(e);
        }

    }




    /**
     * 获取分支集合
     * @param localPath
     * @return
     * @throws GitAPIException
     * @throws IOException
     */
  /*  public List<String> branchList(String localPath) throws GitAPIException, IOException {
        try (Git git = openGit(localPath)) {
            List<Ref> list = git.branchList().setListMode(ListBranchCommand.ListMode.REMOTE).call();
            List<String> all = new ArrayList<>(list.size());
            list.forEach(ref -> {
                String name = ref.getName();
                if (name.startsWith(Constants.R_REMOTES + Constants.DEFAULT_REMOTE_NAME)) {
                    all.add(name.substring((Constants.R_REMOTES + Constants.DEFAULT_REMOTE_NAME).length() + 1));
                }
            });
            return all;
        } catch (TransportException t) {
            t.printStackTrace();
            throw t;
        }
    }*/


    /**
     * clone代码到本地仓库
     * @param remoteUrl  远程路径
     * @param branch     分支名称
     * @param projectPath 本地文件路径
     * @param user 用户名称
     * @param pwd  密码
     * @return
     * @throws InvalidRemoteException
     * @throws TransportException
     * @throws GitAPIException
     */
  /*  public boolean cloneRepository(String remoteUrl, String branch, String projectPath, String user, String pwd) throws InvalidRemoteException, TransportException, GitAPIException {
        File projectDir = new File(projectPath);
        UsernamePasswordCredentialsProvider provider = new UsernamePasswordCredentialsProvider(user, pwd);
        try (Git git = Git.cloneRepository()
                .setURI(remoteUrl)
                .setBranch(branch)
                .setDirectory(projectDir)
                .setCredentialsProvider(provider)
                //.setProgressMonitor(new CloneProgressMonitor())
                .call()) {
        }

        return true;
    }
*/

    /**
     * 初始化git
     * @param remotePath
     * @param localPath
     * @param username
     * @param password
     * @param branch
     * @return
     * @throws GitAPIException
     * @throws IOException
     */
   /* private Git  initGit(String remotePath, String localPath, String username,String password,String branch) throws GitAPIException, IOException {
        UsernamePasswordCredentialsProvider provider = new UsernamePasswordCredentialsProvider(username, password);
        createLocalRepo(localPath);
        Git git =null;
        if (!new File(localPath +"/"+ ".git").exists()) {
            git= Git.cloneRepository().setCredentialsProvider(provider).setURI(remotePath).setBranch(branch)
                    .setDirectory(new File(localPath)).call();
        }else{
            git= Git.open( new File(localPath));
        }
        return git;
    }*/



    public static Git openGit(String localPath) throws IOException {
        Git git = new Git(new FileRepository(localPath + "/.git"));
        return git;
    }



    public static File initFile(String filePath){
        File file = new File(filePath);
        if(! file.exists()){
            file.mkdirs();
            return file;
        }
        return null;
    }



}
